@@ -23,10 +23,10 @@ class Pfs0Stream(BaseFile):
 		self.headerSize = headerSize
 		self._stringTableSize = stringTableSize
 		self.files = []
-		
 		self.actualSize = 0
-
 		self.f.seek(self.headerSize)
+		self.addpos = self.headerSize
+		self.written = False
 
 	def __enter__(self):
 		return self
@@ -36,18 +36,25 @@ class Pfs0Stream(BaseFile):
 		
 	def write(self, value, size = None):
 		super(Pfs0Stream, self).write(value, len(value))
-		if self.tell() > self.actualSize:
-			self.actualSize = self.tell()
+		self.written = True
+		pos = self.tell()
+		if pos > self.actualSize:
+			self.actualSize = pos
 
 	def add(self, name, size, pleaseNoPrint = None):
-		Print.info('[ADDING]     {0} {1} bytes to NSP'.format(name, size), pleaseNoPrint)
-		self.files.append({'name': name, 'size': size, 'offset': self.f.tell()})
-		return self.partition(self.f.tell(), size, n = BaseFile())
+		if self.written:
+			self.addpos = self.tell()
+			self.written = False
+		Print.info(f'[ADDING]     {name} {hex(size)} bytes to PFS0 at {hex(self.addpos)}', pleaseNoPrint)
+		partition = self.partition(self.addpos, size, n = BaseFile())
+		self.files.append({'name': name, 'size': size, 'offset': self.addpos, 'partition': partition})
+		self.addpos += size
+		return partition
 
 	def get(self, name):
 		for i in self.files:
 			if i['name'] == name:
-				return i
+				return i['partition']
 		return None
 
 	def resize(self, name, size):
@@ -62,32 +69,37 @@ class Pfs0Stream(BaseFile):
 			self.seek(0)
 			self.write(self.getHeader())
 			super(Pfs0Stream, self).close()
-			
-	def getHeaderSize(self):
-		stringTable = '\x00'.join(file['name'] for file in self.files)
-		headerSize = 0x10 + len(self.files) * 0x18 + self.stringTableSize
-		return headerSize
+	
+	#0xff => 0x1, 0x100 => 0x20, 0x1ff => 0x1, 0x120 => 0x20
+	def allign0x20(self, n):
+		return 0x20-n%0x20
 	
 	def getStringTableSize(self):
-		stringTable = '\x00'.join(file['name'] for file in self.files)
-		stringTableLen = len(stringTable)
+		stringTableNonPadded = '\x00'.join(file['name'] for file in self.files)+'\x00'
+		headerSizeNonPadded = 0x10 + len(self.files) * 0x18 + len(stringTableNonPadded)
+		stringTableSizePadded = len(stringTableNonPadded) + self.allign0x20(headerSizeNonPadded)
 		if self._stringTableSize == None:
-			self._stringTableSize = stringTableLen
-		if stringTableLen > self._stringTableSize:
-			self._stringTableSize = stringTableLen
+			self._stringTableSize = stringTableSizePadded
+		elif len(stringTableNonPadded) > self._stringTableSize:
+			self._stringTableSize = len(stringTableNonPadded)
 		return self._stringTableSize
 
+	def updateHashHeader(self):
+		pass
+
 	def getFirstFileOffset(self):
 		return self.files[0].offset
 
 	def getHeader(self):
-		stringTable = '\x00'.join(file['name'] for file in self.files)
-		headerSize = 0x10 + len(self.files) * 0x18 + self.getStringTableSize()
+		stringTableNonPadded = '\x00'.join(file['name'] for file in self.files)+'\x00'
+		stringTableSizePadded = self.getStringTableSize()
+		stringTable = stringTableNonPadded + ('\x00'*(stringTableSizePadded-len(stringTableNonPadded)))
+		headerSize = 0x10 + len(self.files) * 0x18 + stringTableSizePadded
 	
 		h = b''
 		h += b'PFS0'
 		h += len(self.files).to_bytes(4, byteorder='little')
-		h += (self.getStringTableSize()).to_bytes(4, byteorder='little')
+		h += (stringTableSizePadded).to_bytes(4, byteorder='little')
 		h += b'\x00\x00\x00\x00'
 		
 		stringOffset = 0
@@ -110,6 +122,7 @@ class Pfs0VerifyStream():
 		self.files = []
 		self.binhash = sha256()
 		self.pos = headerSize
+		self.addpos = headerSize
 		self._stringTableSize = stringTableSize
 
 	def __enter__(self):
@@ -126,59 +139,63 @@ class Pfs0VerifyStream():
 		return self.pos
 
 	def add(self, name, size, pleaseNoPrint = None):
-		Print.info('[ADDING]     {0} {1} bytes to NSP'.format(name, size), pleaseNoPrint)
-		self.files.append({'name': name, 'size': size, 'offset': self.pos})
+		Print.info(f'[ADDING]     {name} {hex(size)} bytes to PFS0 at {hex(self.addpos)}', pleaseNoPrint)
+		self.files.append({'name': name, 'size': size, 'offset': self.addpos})
+		self.addpos += size
 		return self
 
-	def resize(self, name, size):
-		for i in self.files:
-			if i['name'] == name:
-				i['size'] = size
-				return True
-		return False
+	def get(self, name):
+		return self
 
 	def close(self):
 		pass
 	
+	#0xff => 0x1, 0x100 => 0x20, 0x1ff => 0x1, 0x120 => 0x20
+	def allign0x20(self, n):
+		return 0x20-n%0x20
+
 	def getStringTableSize(self):
-		stringTable = '\x00'.join(file['name'] for file in self.files)
-		stringTableLen = len(stringTable)
+		stringTableNonPadded = '\x00'.join(file['name'] for file in self.files)+'\x00'
+		headerSizeNonPadded = 0x10 + len(self.files) * 0x18 + len(stringTableNonPadded)
+		stringTableSizePadded = len(stringTableNonPadded) + self.allign0x20(headerSizeNonPadded)
 		if self._stringTableSize == None:
-			self._stringTableSize = stringTableLen
-		if stringTableLen > self._stringTableSize:
-			self._stringTableSize = stringTableLen
+			self._stringTableSize = stringTableSizePadded
+		elif len(stringTableNonPadded) > self._stringTableSize:
+			self._stringTableSize = len(stringTableNonPadded)
 		return self._stringTableSize
 	
 	def getHash(self):
 		hexHash = self.binhash.hexdigest()
 		return hexHash
 
-	def getHeaderHash(self):
-		stringTable = '\x00'.join(file['name'] for file in self.files)
-		headerSize = 0x10 + len(self.files) * 0x18 + self.getStringTableSize()
+	def updateHashHeader(self):
+		stringTableNonPadded = '\x00'.join(file['name'] for file in self.files)+'\x00'
+		stringTableSizePadded = self.getStringTableSize()
+		stringTable = stringTableNonPadded + ('\x00'*(stringTableSizePadded-len(stringTableNonPadded)))
+		headerSize = 0x10 + len(self.files) * 0x18 + stringTableSizePadded
 	
 		h = b''
 		h += b'PFS0'
 		h += len(self.files).to_bytes(4, byteorder='little')
-		h += (self.getStringTableSize()).to_bytes(4, byteorder='little')
+		h += (stringTableSizePadded).to_bytes(4, byteorder='little')
 		h += b'\x00\x00\x00\x00'
 		
 		stringOffset = 0
-		
 		for f in self.files:
 			h += (f['offset'] - headerSize).to_bytes(8, byteorder='little')
 			h += f['size'].to_bytes(8, byteorder='little')
 			h += stringOffset.to_bytes(4, byteorder='little')
-			h += b'\x00\x00\x00\x00'
-			
+			h += b'\x00\x00\x00\x00'	
 			stringOffset += len(f['name']) + 1
-			
+		
+		if len(self.files) > 0:
+			if self.files[0]['offset'] - headerSize > 0:
+				stringTable += '\x00' * (self.files[0]['offset'] - headerSize)
 		h += stringTable.encode()
 		
-		headerBinhash = sha256()
-		headerBinhash.update(h)
-		headerHexHash = headerBinhash.hexdigest()
-		return headerHexHash
+		headerHex = h.hex()
+		self.binhash.update(h)
+		
 
 
 class Pfs0(BaseFs):
@@ -191,6 +208,13 @@ class Pfs0(BaseFs):
 			#self.offset += sectionStart
 			#self.size -= sectionStart
 
+	#0xff => 0x1, 0x100 => 0x20, 0x1ff => 0x1, 0x120 => 0x20
+	def allign0x20(self, n):
+		return 0x20-n%0x20
+
+	def getPaddedHeaderSize(self):
+		return self._headerSize + self.allign0x20(self._headerSize);
+
 	def getHeaderSize(self):
 		return self._headerSize;
 	
@@ -233,6 +257,7 @@ class Pfs0(BaseFs):
 			nameOffset = self.readInt32() # just the offset
 			name = stringTable[nameOffset:stringEndOffset].decode('utf-8').rstrip(' \t\r\n\0')
 			stringEndOffset = nameOffset
+			Print.info(f'[OPEN  ]     {name} {hex(size)} bytes at {hex(offset)}')
 
 			self.readInt32() # junk data
 
