@@ -6,16 +6,16 @@ from zstandard import ZstdDecompressor
 from nsz.Fs import factory, Type, Pfs0, Hfs0, Nca, Xci
 from nsz.PathTools import *
 from nsz import Header, BlockDecompressorReader, FileExistingChecks
-import enlighten
+import os, enlighten
 
 class VerificationException(Exception):
 	pass
 
-def decompress(filePath, outputDir, removePadding, statusReportInfo, pleaseNoPrint = None):
+def decompress(filePath, outputDir, fixPadding, statusReportInfo, pleaseNoPrint = None):
 	if isNspNsz(filePath):
-		__decompressNsz(filePath, outputDir, removePadding, True, False, False, statusReportInfo, None, pleaseNoPrint)
+		__decompressNsz(filePath, outputDir, fixPadding, True, False, False, statusReportInfo, None, pleaseNoPrint)
 	elif isXciXcz(filePath):
-		__decompressXcz(filePath, outputDir, removePadding, True, False, False, statusReportInfo, None, pleaseNoPrint)
+		__decompressXcz(filePath, outputDir, fixPadding, True, False, False, statusReportInfo, None, pleaseNoPrint)
 	elif isCompressedGameFile(filePath):
 		filePathNca = changeExtension(filePath, '.nca')
 		outPath = filePathNca if outputDir == None else str(Path(outputDir).joinpath(Path(filePathNca).name))
@@ -41,52 +41,79 @@ def decompress(filePath, outputDir, removePadding, statusReportInfo, pleaseNoPri
 		raise NotImplementedError("Can't decompress {0} as that file format isn't implemented!".format(filePath))
 
 
-def verify(filePath, removePadding, raiseVerificationException, raisePfs0Exception, originalFilePath, statusReportInfo, pleaseNoPrint):
+def verify(filePath, fixPadding, raiseVerificationException, raisePfs0Exception, originalFilePath, statusReportInfo, pleaseNoPrint):
 	if isNspNsz(filePath):
-		__decompressNsz(filePath, None, removePadding, False, raiseVerificationException, raisePfs0Exception, originalFilePath, statusReportInfo, pleaseNoPrint)
+		__decompressNsz(filePath, None, fixPadding, False, raiseVerificationException, raisePfs0Exception, originalFilePath, statusReportInfo, pleaseNoPrint)
 	elif isXciXcz(filePath):
-		__decompressXcz(filePath, None, removePadding, False, raiseVerificationException, raisePfs0Exception, originalFilePath, statusReportInfo, pleaseNoPrint)
+		__decompressXcz(filePath, None, fixPadding, False, raiseVerificationException, raisePfs0Exception, originalFilePath, statusReportInfo, pleaseNoPrint)
 
 
 def __decompressContainer(readContainer, writeContainer, fileHashes, write, raiseVerificationException, raisePfs0Exception, statusReportInfo, pleaseNoPrint):
+	CHUNK_SZ = 0x100000
+	if write:
+		for nspf in readContainer:
+			if not nspf._path.endswith('.ncz'):
+				writeContainer.add(nspf._path, nspf.size, pleaseNoPrint)
+			else:
+				newFileName = Path(nspf._path).stem + '.nca'
+				nca_size = __getDecompressedNczSize(nspf)
+				writeContainer.add(newFileName, nca_size, pleaseNoPrint)
+		writeContainer.updateHashHeader()
 	for nspf in readContainer:
-		CHUNK_SZ = 0x100000
-		f = None
+		Print.info('[EXISTS]     {0}'.format(nspf._path), pleaseNoPrint)
 		if not nspf._path.endswith('.ncz'):
 			verifyFile = nspf._path.endswith('.nca') and not nspf._path.endswith('.cnmt.nca')
-			if write:
-				f = writeContainer.add(nspf._path, nspf.size, pleaseNoPrint)
 			hash = sha256()
 			nspf.seek(0)
 			while not nspf.eof():
 				inputChunk = nspf.read(CHUNK_SZ)
 				hash.update(inputChunk)
 				if write:
-					f.write(inputChunk)
+					writeContainer.get(nspf._path).write(inputChunk)
 			if verifyFile:
-				if hash.hexdigest() in fileHashes:
-					Print.info('[VERIFIED]   {0}'.format(nspf._path), pleaseNoPrint)
+				hashHexdigest = hash.hexdigest()
+				if hashHexdigest in fileHashes:
+					Print.info(f'[NCA HASH]   {hashHexdigest}', pleaseNoPrint)
+					Print.info(f'[VERIFIED]   {nspf._path} {hashHexdigest}', pleaseNoPrint)
 				else:
-					Print.info('[CORRUPTED]  {0}'.format(nspf._path), pleaseNoPrint)
+					Print.info(f'[NCA HASH]   {hashHexdigest}', pleaseNoPrint)
+					Print.info(f'[CORRUPTED]  {nspf._path} {hashHexdigest}', pleaseNoPrint)
 					if raiseVerificationException:
 						raise VerificationException("Verification detected hash mismatch!")
-			elif not write:
-				Print.info('[EXISTS]     {0}'.format(nspf._path), pleaseNoPrint)
 			continue
 		newFileName = Path(nspf._path).stem + '.nca'
 		if write:
-			f = writeContainer.add(newFileName, nspf.size, pleaseNoPrint)
-		written, hexHash = __decompressNcz(nspf, f, statusReportInfo, pleaseNoPrint)
-		if write:
-			writeContainer.resize(newFileName, written)
-		if hexHash in fileHashes:
-			Print.info('[VERIFIED]   {0}'.format(nspf._path), pleaseNoPrint)
+			written, hexHash = __decompressNcz(nspf, writeContainer.get(newFileName), statusReportInfo, pleaseNoPrint)
 		else:
-			Print.info('[CORRUPTED]  {0}'.format(nspf._path), pleaseNoPrint)
+			written, hexHash = __decompressNcz(nspf, None, statusReportInfo, pleaseNoPrint)
+		if hexHash in fileHashes:
+			Print.info(f'[NCA HASH]   {hexHash}', pleaseNoPrint)
+			Print.info(f'[VERIFIED]   {nspf._path}', pleaseNoPrint)
+		else:
+			Print.info(f'[NCA HASH]   {hexHash}', pleaseNoPrint)
+			Print.info(f'[CORRUPTED]  {nspf._path}', pleaseNoPrint)
 			if raiseVerificationException:
 				raise VerificationException("Verification detected hash mismatch")
 
 
+def __getDecompressedNczSize(nspf):
+	UNCOMPRESSABLE_HEADER_SIZE = 0x4000
+	nspf.seek(0)
+	header = nspf.read(UNCOMPRESSABLE_HEADER_SIZE)
+	magic = nspf.read(8)
+	if not magic == b'NCZSECTN':
+		raise ValueError("No NCZSECTN found! Is this really a .ncz file?")
+	sectionCount = nspf.readInt64()
+	sections = [Header.Section(nspf) for _ in range(sectionCount)]
+	if sections[0].offset-UNCOMPRESSABLE_HEADER_SIZE > 0:
+		fakeSection = Header.FakeSection(UNCOMPRESSABLE_HEADER_SIZE, sections[0].offset-UNCOMPRESSABLE_HEADER_SIZE)
+		sections.insert(0, fakeSection)
+	nca_size = UNCOMPRESSABLE_HEADER_SIZE
+	for i in range(sectionCount):
+		nca_size += sections[i].size
+	return nca_size
+
+
 def __decompressNcz(nspf, f, statusReportInfo, pleaseNoPrint):
 	UNCOMPRESSABLE_HEADER_SIZE = 0x4000
 	blockID = 0
@@ -95,7 +122,6 @@ def __decompressNcz(nspf, f, statusReportInfo, pleaseNoPrint):
 	currentStep = 'Decompress' if f != None else 'Verifying'
 	if f != None:
 		start = f.tell()
-
 	magic = nspf.read(8)
 	if not magic == b'NCZSECTN':
 		raise ValueError("No NCZSECTN found! Is this really a .ncz file?")
@@ -113,7 +139,7 @@ def __decompressNcz(nspf, f, statusReportInfo, pleaseNoPrint):
 	useBlockCompression = blockMagic == b'NCZBLOCK'
 	blockSize = -1
 	if useBlockCompression:
-		Print.info("[NCZBLOCK]   Using Block decompresion")
+		Print.info(f'[NCZBLOCK]   Using Block decompresion for {nspf._path}')
 		BlockHeader = Header.Block(nspf)
 		blockDecompressorReader = BlockDecompressorReader.BlockDecompressorReader(nspf, BlockHeader)
 	pos = nspf.tell()
@@ -187,64 +213,78 @@ def __decompressNcz(nspf, f, statusReportInfo, pleaseNoPrint):
 	return (0, hexHash)
 
 
-def __decompressNsz(filePath, outputDir, removePadding, write, raiseVerificationException, raisePfs0Exception, originalFilePath, statusReportInfo, pleaseNoPrint):
-	fileHashes = FileExistingChecks.ExtractHashes(filePath)
+def __decompressNsz(filePath, outputDir, fixPadding, write, raiseVerificationException, raisePfs0Exception, originalFilePath, statusReportInfo, pleaseNoPrint):
 	container = factory(filePath)
 	container.open(str(filePath), 'rb')
+	fileHashes = FileExistingChecks.ExtractHashes(container)
 	
 	try:
 		if write:
 			filePathNsp = changeExtension(filePath, '.nsp')
 			outPath = filePathNsp if outputDir == None else str(Path(outputDir).joinpath(Path(filePathNsp).name))
 			Print.info('Decompressing %s -> %s' % (filePath, outPath), pleaseNoPrint)
-			with Pfs0.Pfs0Stream(container.getHeaderSize() if removePadding else container.getFirstFileOffset(), None if removePadding else container.getStringTableSize(), outPath) as nsp:
+			with Pfs0.Pfs0Stream(container.getPaddedHeaderSize() if fixPadding else container.getFirstFileOffset(), None if fixPadding else container.getStringTableSize(), outPath) as nsp:
 				__decompressContainer(container, nsp, fileHashes, True, raiseVerificationException, raisePfs0Exception, statusReportInfo, pleaseNoPrint)
 		else:
-			with Pfs0.Pfs0VerifyStream(container.getHeaderSize() if removePadding else container.getFirstFileOffset(), None if removePadding else container.getStringTableSize()) as nsp:
+			with Pfs0.Pfs0VerifyStream(container.getPaddedHeaderSize() if fixPadding else container.getFirstFileOffset(), None if fixPadding else container.getStringTableSize()) as nsp:
 				__decompressContainer(container, nsp, fileHashes, True, raiseVerificationException, raisePfs0Exception, statusReportInfo, pleaseNoPrint)
-				Print.info("[PFS0 HEAD]  " + nsp.getHeaderHash())
-				Print.info("[PFS0 DATA]  " + nsp.getHash())
+				Print.info("[NSP SHA256] " + nsp.getHash())
 				if originalFilePath != None: 
 					originalContainer = factory(originalFilePath)
-					originalContainer.open(str(originalFilePath), 'rb')
-					with Pfs0.Pfs0VerifyStream(originalContainer.getHeaderSize() if removePadding else originalContainer.getFirstFileOffset(), None if removePadding else container.getStringTableSize()) as originalNsp:
-						__decompressContainer(originalContainer, originalNsp, fileHashes, True, raiseVerificationException, raisePfs0Exception, statusReportInfo, pleaseNoPrint)
-						Print.info("[PFS0 HEAD]  " + originalNsp.getHeaderHash())
-						Print.info("[PFS0 DATA]  " + originalNsp.getHash())
-						if nsp.getHeaderHash() == originalNsp.getHeaderHash():
-							Print.info("[VERIFIED]   PFS0 Header")
-						else:
-							Print.info("[MISSMATCH]  PFS0 Header") 
-							if raisePfs0Exception:
-								raise VerificationException("Verification detected PFS0 hader hash mismatch!")
-						if nsp.getHash() == originalNsp.getHash():
-							Print.info("[VERIFIED]   PFS0 Data")
-						else:
-							Print.info("[MISSMATCH]  PFS0 Data")
-							if raisePfs0Exception:
-								raise VerificationException("Verification detected PFS0 data hash mismatch!")
+					CHUNK_SZ = 0x100000
+					originalHash = sha256()
+					filesize = os.path.getsize(str(originalFilePath))
+					if statusReportInfo == None:
+						BAR_FMT = u'{desc}{desc_pad}{percentage:3.0f}%|{bar}| {count:{len_total}d}/{total:d} {unit} [{elapsed}<{eta}, {rate:.2f}{unit_pad}{unit}/s]'
+						bar = enlighten.Counter(total=filesize//CHUNK_SZ, desc='Verifying', unit="MiB", color='yellow', bar_format=BAR_FMT)
+					blockCount = 0
+					with open(str(originalFilePath), 'rb') as f:
+						while True:
+							data = f.read(CHUNK_SZ)
+							blockCount += 1
+							if statusReportInfo != None:
+								statusReport, id = statusReportInfo
+								statusReport[id] = [blockCount, 0, filesize, 'Verifying']
+							else:
+								bar.count = blockCount
+								bar.refresh()
+							if not data:
+								break
+							originalHash.update(data)
+					originalHashHex = originalHash.hexdigest()
+					if statusReportInfo == None:
+						bar.close()
+					Print.info("[NSP SHA256] " + originalHashHex)
+					if nsp.getHash() == originalHashHex:
+						Print.info("[VERIFIED]   NSP SHA256")
+					else:
+						Print.info("[MISSMATCH]  NSP SHA256")
+						if raisePfs0Exception:
+							raise VerificationException("Verification detected NSP SHA256 hash mismatch!")
 	except BaseException:
 		raise
 	finally:
 		container.close()
 
 
-def __decompressXcz(filePath, outputDir, removePadding, write, raiseVerificationException, raisePfs0Exception, originalFilePath, statusReportInfo, pleaseNoPrint):
-	fileHashes = FileExistingChecks.ExtractHashes(filePath)
+def __decompressXcz(filePath, outputDir, fixPadding, write, raiseVerificationException, raisePfs0Exception, originalFilePath, statusReportInfo, pleaseNoPrint):
 	container = factory(filePath)
 	container.open(str(filePath), 'rb')
-	secureIn = container.hfs0['secure']
 	
 	if write:
 		filePathXci = changeExtension(filePath, '.xci')
 		outPath = filePathXci if outputDir == None else str(Path(outputDir).joinpath(Path(filePathXci).name))
 		Print.info('Decompressing %s -> %s' % (filePath, outPath), pleaseNoPrint)
 		with Xci.XciStream(outPath, originalXciPath = filePath) as xci: # need filepath to copy XCI container settings
-			with Hfs0.Hfs0Stream(xci.hfs0.add('secure', 0, pleaseNoPrint), xci.f.tell()) as secureOut:
-				__decompressContainer(secureIn, secureOut, fileHashes, write, raiseVerificationException, raisePfs0Exception, statusReportInfo, pleaseNoPrint)
-				xci.hfs0.resize('secure', secureOut.actualSize)
+			for partitionIn in container.hfs0:
+				fileHashes = FileExistingChecks.ExtractHashes(partitionIn)
+				hfsPartitionIn = xci.hfs0.add(partitionIn._path, 0x200, pleaseNoPrint)
+				with Hfs0.Hfs0Stream(hfsPartitionIn, xci.f.tell()) as partitionOut:
+					__decompressContainer(partitionIn, partitionOut, fileHashes, write, raiseVerificationException, raisePfs0Exception, statusReportInfo, pleaseNoPrint)
+				xci.hfs0.resize(partitionIn._path, partitionOut.actualSize)
 	else:
-		__decompressContainer(secureIn, None, fileHashes, write, raiseVerificationException, raisePfs0Exception, statusReportInfo, pleaseNoPrint)
+		for partitionIn in container.hfs0:
+			fileHashes = FileExistingChecks.ExtractHashes(partitionIn)
+			__decompressContainer(partitionIn, None, fileHashes, write, raiseVerificationException, raisePfs0Exception, statusReportInfo, pleaseNoPrint)
 
 	container.close()
-
