@@ -14,7 +14,6 @@ from nsz.SolidCompressor import solidCompress
 from traceback import print_exc, format_exc
 from nsz.NszDecompressor import verify as NszVerify, decompress as NszDecompress, VerificationException
 from multiprocessing import cpu_count, freeze_support, Process, Manager
-from nsz.ThreadSafeCounter import Counter
 from nsz.FileExistingChecks import CreateTargetDict, AllowedToWriteOutfile, delete_source_file
 from nsz.ParseArguments import *
 from nsz.PathTools import *
@@ -24,6 +23,12 @@ import enlighten
 import time
 import sys
 
+if hasattr(sys, 'getandroidapilevel'):
+    from nsz.ThreadSafeCounterManager import Counter
+else:
+    from nsz.ThreadSafeCounterSharedMemory import Counter
+
+
 def solidCompressTask(in_queue, statusReport, readyForWork, pleaseNoPrint, pleaseKillYourself, id):
 	while True:
 		readyForWork.increment()
@@ -32,16 +37,17 @@ def solidCompressTask(in_queue, statusReport, readyForWork, pleaseNoPrint, pleas
 		if pleaseKillYourself.value() > 0:
 			break
 		try:
-			filePath, compressionLevel, keepDelta, removePadding, useLongDistanceMode, outputDir, threadsToUse, verifyArg, quickVerify = item
-			outFile = solidCompress(filePath, compressionLevel, keepDelta, removePadding, useLongDistanceMode, outputDir, threadsToUse, statusReport, id, pleaseNoPrint)
+			filePath, compressionLevel, keep, fixPadding, useLongDistanceMode, outputDir, threadsToUse, verifyArg, quickVerify = item
+			outFile = solidCompress(filePath, compressionLevel, keep, fixPadding, useLongDistanceMode, outputDir, threadsToUse, statusReport, id, pleaseNoPrint)
 			if verifyArg:
 				Print.info("[VERIFY NSZ] {0}".format(outFile))
 				try:
-					verify(outFile, removePadding, True, keepDelta, None if quickVerify else filePath, [statusReport, id], pleaseNoPrint)
+					verify(outFile, fixPadding, True, keep, None if quickVerify else filePath, [statusReport, id], pleaseNoPrint)
 				except VerificationException:
 					Print.error("[BAD VERIFY] {0}".format(outFile))
 					Print.error("[DELETE NSZ] {0}".format(outFile))
 					remove(outFile)
+					raise
 		except KeyboardInterrupt:
 			Print.info('Keyboard exception')
 		except BaseException as e:
@@ -53,21 +59,27 @@ def compress(filePath, outputDir, args, work, amountOfTastkQueued):
 	
 	if filePath.suffix == ".xci" and not args.solid or args.block:
 		threadsToUseForBlockCompression = args.threads if args.threads > 0 else cpu_count()
-		outFile = blockCompress(filePath, compressionLevel, args.keep_delta, args.remove_padding, args.long, args.bs, outputDir, threadsToUseForBlockCompression)
+		outFile = blockCompress(filePath, compressionLevel, args.keep, args.fix_padding, args.long, args.bs, outputDir, threadsToUseForBlockCompression)
 		if args.verify:
 			Print.info("[VERIFY NSZ] {0}".format(outFile))
-			verify(outFile, args.remove_padding, True, args.keep_delta, None if args.quick_verify else filePath)
+			try:
+				verify(outFile, args.fix_padding, True, args.keep, None if args.quick_verify else filePath)
+			except VerificationException:
+				Print.error("[BAD VERIFY] {0}".format(outFile))
+				Print.error("[DELETE NSZ] {0}".format(outFile))
+				remove(outFile)
+				raise
 	else:
 		threadsToUseForSolidCompression = args.threads if args.threads > 0 else 3
-		work.put([filePath, compressionLevel, args.keep_delta, args.remove_padding, args.long, outputDir, threadsToUseForSolidCompression, args.verify, args.quick_verify])
+		work.put([filePath, compressionLevel, args.keep, args.fix_padding, args.long, outputDir, threadsToUseForSolidCompression, args.verify, args.quick_verify])
 		amountOfTastkQueued.increment()
 
 
-def decompress(filePath, outputDir, removePadding, statusReportInfo = None):
-	NszDecompress(filePath, outputDir, removePadding, statusReportInfo)
+def decompress(filePath, outputDir, fixPadding, statusReportInfo = None):
+	NszDecompress(filePath, outputDir, fixPadding, statusReportInfo)
 
-def verify(filePath, removePadding, raiseVerificationException, raisePfs0Exception, originalFilePath = None, statusReportInfo = None, pleaseNoPrint = None):
-	NszVerify(filePath, removePadding, raiseVerificationException, raisePfs0Exception, originalFilePath, statusReportInfo, pleaseNoPrint)
+def verify(filePath, fixPadding, raiseVerificationException, raisePfs0Exception, originalFilePath = None, statusReportInfo = None, pleaseNoPrint = None):
+	NszVerify(filePath, fixPadding, raiseVerificationException, raisePfs0Exception, originalFilePath, statusReportInfo, pleaseNoPrint)
 
 err = []
 
@@ -77,6 +89,20 @@ def main():
 		if len(argv) > 1:
 			args = ParseArguments.parse()
 		else:
+			kivyConfigPathObj = Path.home().joinpath('.kivy').joinpath('config.ini')
+			if kivyConfigPathObj.exists():
+				kivyConfigPath = str(kivyConfigPathObj)
+				with open(kivyConfigPath, 'r', encoding='utf-8') as f:
+					lines = f.readlines()
+				brokenKivyConfig = False
+				for i, line in enumerate(lines):
+					if line.startswith('default_font') and 'MPLUS1p-Medium.ttf' in line:
+						brokenKivyConfig = True
+						lines[i] = "default_font = ['Roboto', 'data/fonts/Roboto-Regular.ttf', 'data/fonts/Roboto-Italic.ttf', 'data/fonts/Roboto-Bold.ttf', 'data/fonts/Roboto-BoldItalic.ttf']\n"
+				if brokenKivyConfig:
+					Print.info(f'[INFO   ] Fixing {kivyConfigPath}')
+					with open(kivyConfigPath, 'w', encoding='utf-8') as f:
+						f.writelines(lines)
 			try:
 				from nsz.gui.NSZ_GUI import GUI
 			except ImportError:
@@ -100,7 +126,7 @@ def main():
 		argOutFolder = Path(argOutFolderToPharse).resolve() if args.output else None
 		
 		Print.info('')
-		Print.info('             NSZ v4.5   ,;:;;,')
+		Print.info('             NSZ v4.6   ,;:;;,')
 		Print.info('                       ;;;;;')
 		Print.info('               .=\',    ;:;;:,')
 		Print.info('              /_\', "=. \';:;:;')
@@ -151,8 +177,11 @@ def main():
 			nsp.pack(args.file)
 
 		if args.C:
-			if args.verify and not args.quick_verify and not args.keep_delta:
-				Print.info("Warning: --verify requires --keep-delta when used during compression or it will detect removed NDV0 fragments as errors. For compatibility reasons --quick-verify will be automatically used instead to match the command line argument behavior prior to NSZ v4.3.0.")
+			if args.verify and not args.quick_verify and not args.keep:
+				Print.info("Warning: --verify requires --keep when used during compression or it will detect removed NDV0 fragments as errors. For compatibility reasons --quick-verify will be automatically used instead to match the command line argument behavior prior to NSZ v4.3.0.")
+				args.quick_verify = True
+			if args.verify and not args.quick_verify and args.fix_padding:
+				Print.info("Warning: --verify and --fix-padding are incompatible with each others. For compatibility reasons --quick-verify will be automatically used instead to match the command line argument behavior prior to NSZ v4.6.0.")
 				args.quick_verify = True
 			sourceFileToDelete = []
 			for f_str in args.file:
@@ -223,7 +252,10 @@ def main():
 			barManager.stop()
 
 			for filePath in sourceFileToDelete:
-				delete_source_file(filePath, outFolder)
+				if argOutFolder:
+					delete_source_file(filePath, outFolder)
+				else:
+					delete_source_file(filePath, filePath.parent.absolute())
 
 		if args.D:
 			for f_str in args.file:
@@ -248,7 +280,7 @@ def main():
 								Print.info('{0} with the same file name already exists in the output directory.\n'\
 								'If you want to overwrite it use the -w parameter!'.format(outFile.name))
 								continue
-						decompress(filePath, outFolder, args.remove_padding)
+						decompress(filePath, outFolder, args.fix_padding)
 						if args.rm_source:
 							delete_source_file(filePath, outFolder)
 					except KeyboardInterrupt:
@@ -274,7 +306,7 @@ def main():
 					try:
 						if isGame(filePath):
 							Print.info("[VERIFY {0}] {1}".format(getExtensionName(filePath), filePath.name))
-							verify(filePath, args.remove_padding, True, True)
+							verify(filePath, args.fix_padding, True, True)
 					except KeyboardInterrupt:
 						raise
 					except BaseException as e:
